# arg1: port name, ex: eth0
get_port_speed() {
	local port=$1
	local speed=$(ethtool $port | grep Speed: | awk '{ print $2 }' | sed 's_Mb/s__')
	local duplex=$(ethtool $port | grep Duplex: | awk '{ print $2 }')

	if [ $duplex == "Full" ]; then
		echo "${speed}FD"
	else
		echo "${speed}HD"
	fi
}

port_to_id() {
	local port="$1"
	case "$port" in
		eth0_1) echo "2";;
		eth0_2) echo "3";;
		eth0_3) echo "4";;
		eth0_4) echo "5";;
		eth1) echo "1";;
		*);;
	esac
}

get_dev() {
	local port="$1"
	case "$port" in
		eth0_*) echo "0";;
		eth1) echo "1";;
	esac
}

restart_port() {
	local port=$1

	port_id=$(port_to_id $port)
	dev=$(get_dev $port)
	phy_reg=$(switch_cli dev=$dev GSW_MDIO_DATA_READ nAddressDev=$port_id | grep nData | awk -F '\t' '{print $3}' | awk -F ' ' '{print $1}')

	pwr=$((phy_reg & 0x800))

	# if port is up, power down, maintaining rest of settings
	if [ "$pwr" == "0" ]; then
		write_data=$(printf "0x%X\n" $(($phy_reg + 0x800)))
		# keep phy_reg up to date with what is written to register
		phy_reg=$write_data
		switch_cli dev=$dev GSW_MDIO_DATA_WRITE nAddressDev=$port_id nAddressReg=0x0 nData=$write_data > /dev/null 2>&1
	fi

	# power up, maintaining rest of settings
	write_data=$(printf "0x%X\n" $(($phy_reg - 0x800)))
	switch_cli dev=$dev GSW_MDIO_DATA_WRITE nAddressDev=$port_id nAddressReg=0x0 nData=$write_data > /dev/null 2>&1
}

# arg1: port name, ex: eth0
# arg2: port speed, ex: auto/[10-1000][FD/HD]
# arg3: uci section parsed, ex: LAN3
set_port_speed() {
	local port="$1"
	local setting="$2"
	local cfg="$3"
	local speed=$(echo $setting | tr -dc '0-9')
	local duplex
	local auto

	# full auto is equivalent of 1000FDAUTO, set speed manually
	[ "$setting" == "auto" ] && speed="1000"

	local cur_duplex cur_auto cur_speed
	cur_duplex=$(ethtool $port | grep "Duplex:" | awk '{print $2}' | awk '{print tolower($0)}')
	cur_auto=$(ethtool $port | grep "Auto-negotiation:" | awk '{print $2}' | awk '{print tolower($0)}')
	cur_speed=$(ethtool $port | grep "Speed:" | awk '{print $2}' | tr -dc '0-9')

	port_id=$(port_to_id $port)
	dev=$(get_dev $port)
	phy_reg=$(switch_cli dev=$dev GSW_MDIO_DATA_READ nAddressDev=$port_id | grep nData | awk -F '\t' '{print $3}' | awk -F ' ' '{print $1}')

	pwr=$((phy_reg & 0x800))

	case "$setting" in
		disabled)
			# if port is up, power down, maintaining rest of settings
			if [ "$pwr" == "0" ]; then
				write_data=$(printf "0x%X\n" $(($phy_reg + 0x800)))
				switch_cli dev=$dev GSW_MDIO_DATA_WRITE nAddressDev=$port_id nAddressReg=0x0 nData=$write_data > /dev/null 2>&1
				ifconfig $port down
			fi
			return
		;;
		*)
			# if port is down, power up, maintaining rest of settings
			[ "$pwr" != "0" ] && {
				write_data=$(printf "0x%X\n" $(($phy_reg - 0x800)))
				switch_cli dev=$dev GSW_MDIO_DATA_WRITE nAddressDev=$port_id nAddressReg=0x0 nData=$write_data > /dev/null 2>&1
			}

			ifstatus=$(ip link show $port up)
			[ -z "$ifstatus" ] && ifconfig $port up

			case "$setting" in
				*HD*) duplex="half";;
				*) duplex="full";;
			esac
			case "$setting" in
				auto) auto="on";;
				*AUTO) auto="on";;
				*) auto="off";;
			esac
		;;
	esac

	# if autoneg, duplex and cur speed are the same AND config did, no additional change required
	[ "$cur_auto" = "$auto" -a "$cur_duplex" = "$duplex" -a "$cur_speed" = "$speed" ] && return

	advertise() {
		local speed=$1
		local duplex=$2
		local auto=$3
		local port=$4
		local advertise

		[ "$auto" = "off" ] && return

		case "$speed" in
			1000)
				advertise="15"
				[ "$duplex" = "full" ] && advertise="3f"
			;;
			100)
				advertise="05"
				[ "$duplex" = "full" ] && advertise="0f"
			;;
			10)
				advertise="01"
				[ "$duplex" = "full" ] && advertise="03"
			;;
			*)
				return
			;;
		esac

		advertise="0x0$advertise"
		ethtool -s $port advertise $advertise
	}

	ethtool -s $port speed $speed duplex $duplex autoneg $auto
	advertise $speed $duplex $auto $port
}

# arg1: port name, ex: eth0
get_port_status() {
	local port="$1"
	local auto=$(ethtool $port | grep Auto-negotiation: | awk '{ print $2 }')

	if [ $auto == "on" ]; then
		echo "auto"
	else
		get_port_speed $port
	fi
}

# arg1: port name, ex: eth0
get_port_number() {
	:
}

# arg1: port name, ex: eth0
get_port_flag(){
	:
}

# arg1: sfp port name, ex: eth0
get_sfp_flag(){
	:
}

# arg1: port name, ex: eth0
# arg2: 0 or 1
set_port_pause() {
	:
}

# arg1: fiber port name, ex: eth0
set_fiber_speed() {
	:
}
