# arg1: ifname, ex: eth0.1
get_network_of() {

	find_network() {
		local config="$1"
		local iface="$2"
		local net=
		ifname="$(uci get network.$config.ifname)"
		for ifc in $ifname; do
			if [ "$ifc" == "$iface" ]; then
				net=$config
				break
			fi
		done
		[ -n "$net" ] && echo $net
	}

	config_load network
	config_foreach find_network interface $1
}

# arg1: port name, ex: eth0
interfacename() {
	local PORT_ORDER=$(db get hw.board.ethernetPortOrder)
	local PORT_NAMES=$(db get hw.board.ethernetPortNames)
	local cnt=1
	local idx=1

	# get index of interface name
	for i in $PORT_ORDER; do
		if [ "$i" == "$1" ]; then
			break;
		fi
		idx=$((idx+1))
	done

	# get port name from index
	for i in $PORT_NAMES; do
		if [ "$cnt" == "$idx" ]; then
			echo $i
			break;
		fi
		cnt=$((cnt+1))
	done

	# for wifi use default
	uci show wireless | grep ifname | grep -wq "$1" && echo "WLAN"
}

# arg1: fiber port name, ex: eth0
fibername() {
	local FIBER_ORDER=$(db get hw.board.fiberPortOrder)
	local FIBER_NAMES=$(db get hw.board.fiberPortNames)
	local cnt=1
	local idx=1

	# get index of interface name
	for i in $FIBER_ORDER; do
		if [ "$i" == "$1" ]; then
			break;
		fi
		idx=$((idx+1))
	done

	# get port name from index
	for i in $FIBER_NAMES; do
		if [ "$cnt" == "$idx" ]; then
			echo $i
			break;
		fi
		cnt=$((cnt+1))
	done
}

# function wait_for_dns
# arg1: url
# arg2: wait_time (seconds)
# call example: wait_for_dns http://user:password@example.com/path/firmware.y2 30
# waits for dnsmasq to start
wait_for_dns() {
	local url=$1
	local wait_time=${2:-30}
	local wait_interval=2

	[ -n "$url" ] || return 1

	url=${url#*://}		# remove http://
	url=${url%%/*}		# remove /path/firmware.y2
	url=${url#*@}		# remove user:password@
			# url is only example.com

	echo -n "Waiting for dns ..."
	while [ true ] ; do
		nslookup $url &> /dev/null # try DNS lookup for url
		[ $? == 0 ] && break
		sleep $wait_interval
		echo -n "."
		wait_time=$((wait_time - wait_interval))
		[ "$wait_time" -le "0" ] && { echo " failed"; return 1;} # timer expired
	done
	echo " ok"
	return 0
}
# end of wait_for_dns

# checks whether any of the uplink
# ports are physically connected
uplink_phy_connected() {
	local wanports="$(db -q get hw.board.ethernetWanPort) $(db -q get hw.board.vdslWanPort) $(db -q get hw.board.adslWanPort)"
	local ret=1

	for port in $wanports; do
		if [ "$(cat /sys/class/net/$port/operstate 2>/dev/null)" == "up" ]; then
			ret=0
			break
		fi
	done

	return $ret
}

